@@ -1,4 +1,4 @@
-set rcsid {$Id: datatype3.tcl,v 1.12 2006/01/26 13:25:08 danielk1977 Exp $}
+set rcsid {$Id: datatype3.tcl,v 1.17 2007/06/20 16:13:23 drh Exp $}
 source common.tcl
 header {Datatypes In SQLite Version 3}
 puts {
@@ -87,6 +87,7 @@ following type affinities:</P>
 	<LI>TEXT</LI>
 	<LI>NUMERIC</LI>
 	<LI>INTEGER</LI>
+        <LI>REAL</li>
 	<LI>NONE</LI>
 </UL>
 
@@ -108,6 +109,12 @@ floating point component (or text value that converts to such) is
 inserted it is converted to an integer and stored using the INTEGER
 storage class.</P>
 
+<P>A column with REAL affinity behaves like a column with NUMERIC
+affinity except that it forces integer values into floating point
+representation.  (As an optimization, integer values are stored on
+disk as integers in order to take up less space and are only converted
+to floating point as the value is read out of the table.)</P>
+
 <P>A column with affinity NONE does not prefer one storage class over
 another.  It makes no attempt to coerce data before
 it is inserted.</P>
@@ -129,6 +136,10 @@ of the column, according to the following rules:</P>
          contains the string &quot;BLOB&quot; or if
         no datatype is specified then the column has affinity NONE.</P>
 
+        <LI><P>If the datatype for a column
+        contains any of the strings &quot;REAL&quot;, &quot;FLOA&quot;,
+        or &quot;DOUB&quot; then the column has REAL affinity</P>
+
 	<LI><P>Otherwise, the affinity is NUMERIC.</P>
 </OL>
 
@@ -156,6 +167,7 @@ INSERT INTO t1 VALUES(500.0, 500.0, 500.0, 500.0);
 </PRE>
 </blockquote>
 
+<a name="comparisons">
 <h3>3. Comparison Expressions</h3>
 
 <P>Like SQLite version 2, version 3
@@ -185,18 +197,20 @@ two values being compared, according to the following rules:</P>
 classes (INTEGER and REAL) and TEXT before performing a comparison.
 For binary comparisons, this is done in the cases enumerated below.
 The term "expression" used in the bullet points below means any
-SQL scalar expression or literal other than a column value.</P>
+SQL scalar expression or literal other than a column value.  Note that
+if X and Y.Z are a column names, then +X and +Y.Z are considered
+expressions.</P>
 <UL>
 	<LI><P>When a column value is compared to the result of an
 	expression, the affinity of the column is applied to the result of
 	the expression before the comparison takes place.</P>
 
 	<LI><P>When two column values are compared, if one column has
-	INTEGER or NUMERIC affinity and the other does not, the NUMERIC
-	affinity is applied to any values with storage class TEXT extracted
-	from the non-NUMERIC column.</P>
+	INTEGER or REAL or NUMERIC affinity and the other does not, 
+        then NUMERIC affinity is applied to any values with storage
+        class TEXT extracted from the non-NUMERIC column.</P>
 
-	<LI><P>When the results of two expressions are compared, the no
+	<LI><P>When the results of two expressions are compared, no
         conversions occur.  The results are compared as is.  If a string
         is compared to a number, the number will always be less than the
         string.</P>
@@ -214,8 +228,12 @@ similar manner to "a = b"). For example if 'b' is a column value
 and 'a' is an expression, then the affinity of 'b' is applied to 'a'
 before any comparisons take place.</P>
 
-<P>SQLite treats the expression "a IN (x, y, z)" as equivalent to "a = z OR
-a = y OR a = z".
+<P>SQLite treats the expression "a IN (x, y, z)" as equivalent to "a = +x OR
+a = +y OR a = +z".  The values to the right of the IN operator (the "x", "y",
+and "z" values in this example) are considered to be expressions, even if they
+happen to be column values.  If the value of the left of the IN operator is
+a column, then the affinity of that column is used.  If the value is an
+expression then no conversions occur.
 </P>
 
 <h4>3.1 Comparison Example</h4>
@@ -312,13 +330,12 @@ instead of memcmp().
 </p>  
 <p>
 Aside from the default collation sequence BINARY, implemented using
-memcmp(), SQLite features two extra built-in collation sequences 
-intended for testing purposes, NOCASE and REVERSE:
+memcmp(), SQLite features one extra built-in collation sequences 
+intended for testing purposes, the NOCASE collation:
 </p>  
 <UL>
 	<LI><b>BINARY</b> - Compares string data using memcmp(), regardless
                             of text encoding.</LI>
-	<LI><b>REVERSE</b> - Collate in the reverse order to BINARY. </LI>
 	<LI><b>NOCASE</b> - The same as binary, except the 26 upper case
 			    characters used by the English language are
 			    folded to their lower case equivalents before
@@ -346,7 +363,8 @@ column, then the default collation type of the column determines the
 collation sequence to use for the comparison. If both operands are columns,
 then the collation type for the left operand determines the collation
 sequence used. If neither operand is a column, then the BINARY collation
-sequence is used.
+sequence is used.  For the purposes of this paragraph, a column name
+preceded by one or more unary "+" operators is considered a column name.
 </p>  
 
 <p>
