@@ -1,7 +1,7 @@
 #
 # Run this Tcl script to generate the formatchng.html file.
 #
-set rcsid {$Id: formatchng.tcl,v 1.15 2006/01/11 01:08:34 drh Exp $ }
+set rcsid {$Id: formatchng.tcl,v 1.20 2007/09/03 20:32:45 drh Exp $ }
 source common.tcl
 header {File Format Changes in SQLite}
 puts {
@@ -189,7 +189,7 @@ occurred since version 1.0.0:
 </tr>
 <tr>
   <td valign="top">3.0.8 to 3.1.0</td>
-  <td valign="top">2004-Jan-21</td>
+  <td valign="top">2005-Jan-21</td>
   <td><p>Version 3.1.0 adds support for
   <a href="pragma.html#pragma_auto_vacuum">autovacuum mode</a>.
   Prior versions of SQLite will be able to read an autovacuumed
@@ -200,7 +200,7 @@ occurred since version 1.0.0:
 </tr>
 <tr>
   <td valign="top">3.1.6 to 3.2.0</td>
-  <td valign="top">2004-Mar-19</td>
+  <td valign="top">2005-Mar-19</td>
   <td><p>Version 3.2.0 adds support for the 
   <a href="lang_altertable.html">ALTER TABLE ADD COLUMN</a>
   command.  A database that has been modified by this command can
@@ -214,11 +214,55 @@ occurred since version 1.0.0:
 <tr>
   <td valign="top">3.2.8 to 3.3.0</td>
   <td valign="top">2006-Jan-10</td>
-  <td><p>Version 3.3.0 uses a new encoding for binary values that
-  uses less disk space.  Version 3.3.0 can read and write database
+  <td><p>Version 3.3.0 adds support for descending indices and
+  uses a new encoding for boolean values that requires
+  less disk space.  Version 3.3.0 can read and write database
   files created by prior versions of SQLite.  But prior versions
   of SQLite will not be able to read or write databases created
   by Version 3.3.0</p>
+  <p>If you need backwards and forwards capatibility, you can
+  compile with -DSQLITE_DEFAULT_FILE_FORMAT=1.  Or at runtime
+  you can say "PRAGMA legacy_file_format=ON" prior to creating
+  a new database file</p>
+  <p>Once a database file is created, its format is fixed.  So
+  a database file created by SQLite 3.2.8 and merely modified
+  by version 3.3.0 or later will retain the old format.  Except,
+  the VACUUM command recreates the database so running VACUUM
+  on 3.3.0 or later will change the file format to the latest
+  edition.</p>
+  </td>
+</tr>
+<tr>
+  <td valign="top">3.3.6 to 3.3.7</td>
+  <td valign="top">2006-Aug-12</td>
+  <td><p>The previous file format change has caused so much
+  grief that the default behavior has been changed back to 
+  the original file format.  This means that DESC option on
+  indices is ignored by default that the more efficient encoding
+  of boolean values is not used.  In that way, older versions
+  of SQLite can read and write databases created by newer
+  versions.  If the new features are desired, they can be
+  enabled using pragma: "PRAGMA legacy_file_format=OFF".</p>
+  <p>To be clear: both old and new file formats continue to
+  be understood and continue to work.  But the old file format
+  is used by default instead of the new.  This might change
+  again in some future release - we may go back to generating
+  the new file format by default - but probably not until
+  all users have upgraded to a version of SQLite that will
+  understand the new file format.  That might take several
+  years.</p></td>
+</tr>
+<tr>
+  <td valign="top">3.4.2 to 3.5.0</td>
+  <td valign="top">2007-Sep-3</td>
+  <td><p>The design of the OS interface layer was changed for
+  release 3.5.0.  Applications that implemented a custom OS
+  interface will need to be modified in order to upgrade.
+  There are also some subtly different semantics a few obscure
+  APIs.  An <a href="34to35.html">article</a> is avilable which
+  describing the changes in detail.</p>
+
+  <p>The on-disk file format is unchanged.</p>
   </td>
 </tr>
 </table>
