@@ -9,6 +9,55 @@ proc newsitem {date title text} {
   puts "<hr width=\"50%\">"
 }
 
+newsitem {2007-Aug-13} {Version 3.4.2} {
+  While stress-testing the 
+  <a href="capi3ref.html#sqlite3_soft_heap_limit">soft_heap_limit</a>
+  feature, a bug that could lead to
+  <a href="http://www.sqlite.org/cvstrac/wiki?p=DatabaseCorruption">database
+  corruption</a> was <a href="http://www.sqlite.org/cvstrac/tktview?tn=2565">
+  discovered and fixed</a>.
+  Though the consequences of this bug are severe, the chances of hitting 
+  it in a typical application are remote.  Upgrading is recommended
+  only if you use the 
+  <a href="capi3ref.html#sqlite3_soft_heap_limit">sqlite3_soft_heap_limit</a>
+  interface.
+}
+
+newsitem {2007-Jly-20} {Version 3.4.1} {
+  This release fixes a bug in <a href="lang_vacuum.html">VACUUM</a> that
+  can lead to <a href="http://www.sqlite.org/cvstrac/wiki?p=DatabaseCorruption">
+  database corruption</a>.  The bug was introduced in version 
+  <a href="changes.html#version_3_3_14">3.3.14</a>.
+  Upgrading is recommended for all users.  Also included are a slew of
+  other more routine
+  <a href="changes.html#version_3_4_1">enhancements and bug fixes</a>.
+}
+
+newsitem {2007-Jun-18} {Version 3.4.0} {
+  This release fixes two separate bugs either of which 
+  can lead to database corruption.  Upgrading
+  is strongly recommended.  If you must continue using an older version
+  of SQLite, please at least read about how to avoid these bugs
+  at
+  <a href="http://www.sqlite.org/cvstrac/wiki?p=CorruptionFollowingBusyError">
+  CorruptionFollowingBusyError</a> and
+  <a href="http://www.sqlite.org/cvstrac/tktview?tn=2418">ticket #2418</a>
+  <p>
+  This release also adds explicit <a href="limits.html">limits</a> on the
+  sizes and quantities of things SQLite will handle.  The new limits might
+  causes compatibility problems for existing applications that
+  use excessively large strings, BLOBs, tables, or SQL statements. 
+  The new limits can be increased at compile-time to work around any problems
+  that arise.  Nevertheless, the version number of this release is
+  3.4.0 instead of 3.3.18 in order to call attention to the possible
+  incompatibility.
+  </p>
+  There are also new features, including
+  <a href="capi3ref.html#sqlite3_blob_open">incremental BLOB I/O</a> and
+  <a href="pragma.html#pragma_incremental_vacuum">incremental vacuum</a>.
+  See the <a href="changes.html#version_3_4_0">change log</a> 
+  for additional information.
+}
 
 newsitem {2007-Apr-25} {Version 3.3.17} {
   This version fixes a bug in the forwards-compatibility logic of SQLite
@@ -457,4 +506,4 @@ newsitem {2004-Apr-23} {Work Begins On SQLite Version 3} {
   Plans are to continue to support SQLite version 2.8 with
   bug fixes.  But all new development will occur in version 3.0.
 }
-footer {$Id: oldnews.tcl,v 1.22 2007/08/13 16:15:29 drh Exp $}
+footer {$Id: oldnews.tcl,v 1.24 2007/11/05 18:11:18 drh Exp $}
